module ToricAmplitudes

# Author:                 Simon Telen
# Date:                   March 27, 2025
# Short description:      This code supports the paper "Toric Amplitudes and Universal Adjoints" by Simon Telen. 


using Oscar
using IterTools

export nb_cone, to_ind, get_linspaces, get_Adj, get_Amp, sing_locus, irrel, fano_charts, fano, my_associated_primes

# compute the neighboring rays of a cone Δ in the fan Σ
function nb_cone(τ,Σ)   
    cns = cones(Σ,Oscar.dim(τ)+1)
    nbcns = cns[findall(c->is_subset(rays(τ),rays(c)), cns)]
    cands = unique(vcat([rays(a) for a in nbcns]...))
    cands[findall(cnd->!(cnd in rays(τ)),cands)]
end

# convert a list of rays to a list of indices according to the ordering of rays(Σ) 
function to_ind(rys,Σ)
    [findfirst(r->rr == r, rays(Σ)) for rr in rys]
end

# compute linear spaces contained in the universal adjoint hypersurface of P
# this returns two lists: lst1 corresponds to Proposition 6.3, lst2 corresponds to Proposition 5.3
function get_linspaces(Σ,U; x = [])
    n,d = size(U)
    allrays = rays(Σ)
    if isempty(x)
        RΣ, x = polynomial_ring(QQ,:x=>1:length(allrays))
    else
        RΣ = parent(x[1])
    end
    lst1 =[]
    for k = 1:Oscar.dim(Σ)-1
        println(k)
        cns = cones(Σ,k)
        for c in cns 
            if length(nb_cone(c,Σ)) == d-k+1
                indset = [to_ind(rays(c),Σ); to_ind(nb_cone(c,Σ),Σ)]
                MM = reshape(hcat(RΣ.(U[indset,:])...,x[indset]...),d+1,d+1)
                MM = matrix_space(RΣ,size(MM)...)(MM)
                push!(lst1,ideal([det(MM);x[to_ind(rays(c),Σ)]]))
            end
        end
    end
    B = irrelevant_ideal(normal_toric_variety(Σ))
    phi = hom(base_ring(B),RΣ,x)
    B = phi(B)
    lst2 = minimal_primes(B)
    return unique(lst1), lst2
end


# Compute the universal adjoint of a simplicial fan 
function get_Adj(Σ,U; x = [])
    n,d = size(U)
    if !is_simplicial(Σ)
        println("wanted simplicial fan")
        return []
    end
    if isempty(x)
        RΣ, x = polynomial_ring(QQ,:x=>1:length(rays(Σ)))
    end
    Σd = cones(Σ,d)
    rayvec = [collect(rays(σ)) for σ in Σd]
    inds = []
    for j = 1:length(rayvec)
        r = rayvec[j]
        push!(inds,[findfirst(i->Rational{Int}.(U[i,:])==r[k],1:n) for k = 1:length(r)])
    end
    MS = matrix_space(QQ,d,d)
    sum([abs(det(MS(U[ii,:])))*prod(x[setdiff(1:n,ii)]) for ii in inds])
end

# Compute the toric amplitude of a simplicial fan 
# The first output is a list of terms, the second output is their sum (on common denominator)
function get_Amp(Σ,U; x = [])
    n,d = size(U)
    if !is_simplicial(Σ)
        println("wanted simplicial fan")
        return []
    end
    if isempty(x)
        RΣ, x = polynomial_ring(QQ,:x=>1:length(rays(Σ)))
    end
    K = fraction_field(base_ring(x[1]))
    Σd = cones(Σ,d)
    rayvec = [collect(rays(σ)) for σ in Σd]
    inds = []
    for j = 1:length(rayvec)
        r = rayvec[j]
        push!(inds,[findfirst(i->Rational{Int}.(U[i,:])==r[k],1:n) for k = 1:length(r)])
    end
    MS = matrix_space(QQ,d,d)
    Ampterms = [Rational{Int64}(abs(det(MS(U[ii,:]))))//prod(x[ii]) for ii in inds]
    return Ampterms, sum(Ampterms)
end

# Compute the ideal defining the singular locus of F = 0. If dec = true,
# then the function tries to decompose the singular locus into minimal primes. 
function sing_locus(F; dec = false)
    x = vars(F)
    I = ideal([derivative(F,xx) for xx in x])
    if dec
        MP = minimal_primes(I)
        return MP
    else
        return I
    end
end

# Compute the irrelevant ideal of Σ
function irrel(Σ; x = [])
    B = irrelevant_ideal(normal_toric_variety(Σ))
    if !isempty(x)
        phi = hom(base_ring(B),parent(x[1]),x)
        B = phi(B)
    end
    return B
end

# compute the ideal of an incidence variety defining the k-th Fano scheme of F = 0
# works chart by chart
function fano_charts(F,k)
    n = length(vars(F))
    pluckinds = collect(IterTools.subsets(1:n,k+1))
    A,a,u,p = polynomial_ring(QQ,:a=>(1:k+1,k+2:n), :u=>1:1, :p=>pluckinds)
    all_ideals = []
    for PII in pluckinds
        println("chart: $(PII)")
        amtx = matrix_space(A,k+1,n)(zeros(A,k+1,n))
        amtx[:,setdiff(1:n,PII)] = matrix_space(A,size(a)...)(a)
        amtx[:,PII] = identity_matrix(A,k+1)
        Grassparam = []
        for i = 1:length(pluckinds)
            PI = pluckinds[i]
            push!(Grassparam,u[1]*det(amtx[:,PI])-p[i])
        end
        S, t = polynomial_ring(A,:t=>1:k+1)
        argu = transpose(matrix_space(S,1,length(t))(transpose(t))*amtx)
        FF = Oscar.evaluate(F,[aa for aa in argu][:])
        Fano1 = ideal([c[1] for c in coefficients_and_exponents(FF)])
        J = Fano1 + ideal(Grassparam)
        push!(all_ideals,eliminate(J,1:length(vcat(a[:],u))))
    end
    P, p = polynomial_ring(QQ,:p=>pluckinds)
    phi = hom(A,P,[zeros(P,length(a[:])+1);p])
    return [phi(I) for I in all_ideals]
end

# Compute the Fano scheme of k-planes contained in the hypersurface F = 0
function fano(F,k)
    FC = fano_charts(F,k)
    unique(vcat([minimal_primes(fc) for fc in FC]...))
end

# Associated primes of an ideal I 
function my_associated_primes(I)
    PD = primary_decomposition(I)
    [pd[2] for pd in PD]
end

end # module ToricAmplitudes


