# Author:                 Simon Telen
# Date:                   March 27, 2025
# Short description:      This code supports the paper "Toric Amplitudes and Universal Adjoints" by Simon Telen. 

# To use this code, make sure you are in the folder ToricAmplitudes. 
# You can check this by accessing the shell in a julia terminal by clicking ";" and then using "pwd":
# shell> pwd
# If this does not show the path to ToricAmplitudes, move to the correct folder by using "cd":
# shell> cd .../ToricAmplitudes
# Hit backspace to exit the shell. 

# Next, use the following commands: 
# using Pkg
# Pkg.activate(".")
# Pkg.instantiate()

using ToricAmplitudes
using Oscar

#-------------------------------------------------------------------------
# Section 2.1: The universal adjoint quadric of a quadrilateral

U = [1 0; 0 1; -1//2 -1; 1 -1//2] # choose a non-degenerate ray matrix for the fan of a quadrilateral 
IM = incidence_matrix([[1,2],[2,3],[3,4],[4,1]]); # This indicates which rows of U generate a maximal cone in our fan 
Σ = polyhedral_fan(IM, U)
R, x = polynomial_ring(QQ, :x=>1:4)

# Compute the universal adjoint
AdjΣ = get_Adj(Σ,U)
# The Fano scheme of lines is computed as follows: 
F1 = fano(AdjΣ,1)

#-------------------------------------------------------------------------
# Section 2.2: The Segre cubic of a pentagon

U = [1 0; 0 1; -1 1; -1 0; 0 -1] # This is U from Example 1.1
IM = incidence_matrix([[1,2],[2,3],[3,4],[4,5],[5,1]]);
Σ = polyhedral_fan(IM, U)

# Compute the toric amplitude
Ampterms, Amp = get_Amp(Σ,U)
# Compute the universal adjoint
AdjΣ = get_Adj(Σ,U)
# Fano schemes 
F1 = fano(AdjΣ,1)
F2 = fano(AdjΣ,2)
# Singular locus 
SingAΣ = sing_locus(AdjΣ; dec = true)

#-------------------------------------------------------------------------
# Section 2.3: The three-dimensional associahedron

U = transpose([-1 0 0 1 1 1 0 0 0; 0 -1 0 -1 0 0 1 1 0; 0 0 -1 0 -1 0 -1 0 1])
IM = incidence_matrix([[3,5,7],[1,2,9],[1,2,3],[1,3,7],[1,7,8],[2,3,4],[1,8,9],[2,4,9],[3,4,5],[4,5,6],[4,6,9],[5,6,7],[6,7,8],[6,8,9]])
Σ = polyhedral_fan(IM, U)
f_vector(Σ) # 9, 21, 14
R, x = polynomial_ring(QQ,["x13","x14","x15","x24","x25","x26","x35","x36","x46"])
R, x = polynomial_ring(QQ,["x_{13}","x_{14}","x_{15}","x_{24}","x_{25}","x_{26}","x_{35}","x_{36}","x_{46}"])

# Compute the toric amplitude
Ampterms, Amp = get_Amp(Σ,U; x = x)
# Compute the universal adjoint
AdjΣ = get_Adj(Σ,U; x = x)
# Compute the Fano scheme of 6-planes
F6 = fano(AdjΣ,6)
# The intersection of the singular locus with Z(Σ) has 133 components 
# To compute the irrelevant ideal, we must order the variables compatible with the output of rays(Σ)
srt = [findfirst(i->U[i,:] == r,1:9) for r in rays(Σ)]
BΣ = irrel(Σ; x = x[srt])
SingAΣ = sing_locus(AdjΣ; dec = false)
longlist = vcat([minimal_primes(SingAΣ+I) for I in minimal_primes(BΣ)]...)
sing_inZΣ = unique(longlist) # some components are found multiple times. There are 133 different ones. 
dims = dim.(sing_inZΣ) # There are components of dimension 5 and 6. Projectively, these have dimension 4 and 5. 
dimgroups = [findall(k->k==i, dims) for i = 1:9]
deggroups = [findall(k->degree(k)==i, sing_inZΣ[dimgroups[5]]) for i in 1:10] # The components of projective dimension 4 have degree 1, 3 or 7
# We now compute the components of the singular locus which are not contained in Z(Σ)
sing_notinZΣ = minimal_primes(saturation(SingAΣ,BΣ))
dim.(sing_notinZΣ)
degree.(sing_notinZΣ)

#-------------------------------------------------------------------------
# Example 3.6: A non-complete fan

U = transpose([1 1 1 1 1 1; 1 0 -1 0 0 -1; 0 1 0 -1 0 -1])
n,d = size(U)
IM = incidence_matrix([[1,2,5],[2,3,5],[3,4,5],[1,4,5],[3,6],[4,6]]);
Σ = polyhedral_fan(IM, U)
# Compute the universal adjoint
AdjΣ = get_Adj(Σ,U)
factor(AdjΣ)

#-------------------------------------------------------------------------
# Example 3.11: A complete, non-projective fan

U = [-1 0 0; 0 -1 0; 0 0 -1; 1 1 1; 1 1 0; 0 1 1; 1 0 1]
n,d = size(U)
IM = incidence_matrix([[4,5,6],[1,5,6],[1,3,5],[4,5,7],[3,5,7],[2,3,7],[4,6,7],[2,6,7],[1,2,6],[1,2,3]]);
Σ = polyhedral_fan(IM, U)
is_complete(Σ)
is_projective(normal_toric_variety(Σ))

# Compute the universal adjoint
AdjΣ = get_Adj(Σ,U)
# Check that it vanishes on the image of U
T,y = polynomial_ring(QQ,:y=>1:3)
evaluate(AdjΣ,U*y)

#-------------------------------------------------------------------------
# Example 4.7: Warren's adjoint

# We start from the normal fan of a pentagon
U = [1 0; 0 1; -1 1; -1 0; 0 -1] # This is U from Example 1.1
IM = incidence_matrix([[1,2],[2,3],[3,4],[4,5],[5,1]]);
Σ = polyhedral_fan(IM, U)

# Compute the universal adjoint
AdjΣ = get_Adj(Σ,U)
# Now substitute the edge lines to obtain the adjoint
T, y = polynomial_ring(QQ,:y=>1:2)
z = [1;1;1;1;1]
evaluate(AdjΣ,U*y+z)

# We repeat this computation for an unbounded polyhedron
U = [2 5; 1 1; 1 0; 1 -1; 2 -5] # This is U from Example 1.1
IM = incidence_matrix([[1,2],[2,3],[3,4],[4,5]]);
Σ = polyhedral_fan(IM, U)

# Compute the universal adjoint
AdjΣ = get_Adj(Σ,U)
# Now substitute the edge lines to obtain the adjoint
T, y = polynomial_ring(QQ,:y=>1:2)
z = [10,3,2,3,10]
evaluate(AdjΣ,U*y+z)

#-------------------------------------------------------------------------
# Example 6.6: A simple 3-polytope

U = transpose([-8 10 -4 2 0 0; -11 12 -3 0 2 0; -7 6 -1 0 0 2])
z = [479; 336; 69; 78; 208; 78]
P = polyhedron(-U,z)
facets(P) # this confirms that the command above defines the polytope we want
Σ = normal_fan(P)

# Compute the universal adjoint
AdjΣ = get_Adj(Σ,U)
# Compute the linear spaces from Propositions 5.3 and 6.3
lst1, lst2 = get_linspaces(Σ,U)
# The linear spaces from Proposition 5.3 are in lst1, and those of Proposition 6.3 are in lst2
# By Theorem 5.6, the intersection of these ideals has only one generator in degree 3 - the universal adjoint
intersect(lst1...,lst2...)

# Specializing to Warren's adjoint 
T, y = polynomial_ring(QQ,:y=>1:3)
# Our polytope loses a facet for the following choice of z: 
z0 = [7;0;1;0;0;2]
adj = evaluate(AdjΣ,U*y+z0)
# The adjoint splits off a linear form, as predicted in the proof of Proposition 6.5. 
factor(adj)
# The polytope degenerates also for 
z1 = [33;-26;9;0;0;0]
adj = evaluate(AdjΣ,U*y+z1)
# It has a non-simplicial vertex, at which the adjoint must vanish: 
vertices(polyhedron(-U,z1))
evaluate(adj,[0,3,0])

# In Example 7.10, it is concluded that the singular locus of the universal adjoint
# is contained in Z(Σ). We check this here. 
SingAΣ = sing_locus(AdjΣ)
BΣ = irrel(Σ; x = vars(AdjΣ))
saturation(SingAΣ,BΣ)

#-------------------------------------------------------------------------
# Example 7.5: the hexagon

U = [1 0; 1 1; 0 1; -1 0; -1 -1; 0 -1] 
IM = incidence_matrix([[1,2],[2,3],[3,4],[4,5],[5,6],[1,6]])
Σ = polyhedral_fan(IM, U)

# Compute the universal adjoint
AdjΣ = get_Adj(Σ,U)
# The divisoral fano scheme consists of 15 components 
F2 = fano(AdjΣ,3)
# Here 15 = 9 + 6. There are 9 components comming from the irrelevant ideal 
BΣ = irrel(Σ; x = vars(AdjΣ))
minimal_primes(BΣ)
# and the six other components come from the six edges (Theorem 5.6). 
# These three-planes are in the first list generated by the command get_linspaces: 
lst1, lst2 = get_linspaces(Σ,U; x = vars(AdjΣ))
# The second list equals minimal_primes(BΣ). 
# AdjΣ is the only quartic interpolating these two lists: 
J = intersect(lst1...,lst2...)
# J has only one generator of degree four. 
# The singular locus is computed as follows:
SingAΣ = sing_locus(AdjΣ; dec = true)
# It is contained in the zero locus of BΣ by Theorem 7.11: 
saturation(intersect(SingAΣ...), BΣ)